import logging
from kuaishou.vod.core.exception.kuaishou_vod_sdk_exception import KuaishouVodSDKException
from requests import Session

from kuaishou.vod.core.http.content_type import APPLICATION_JSON, APPLICATION_FORM

try:
    from http.client import HTTPConnection, BadStatusLine, HTTPSConnection
    from urllib.parse import urlparse
except ImportError:
    from httplib import HTTPConnection, BadStatusLine, HTTPSConnection
    from urlparse import urlparse

LOG = logging.getLogger("MediaCloudPythonSDK")


class RequestInternal(object):
    def __init__(self, host="", method="", uri="", headers=None, data="", content_type='application/json'):
        if headers is None:
            headers = {}
        self.host = host
        self.method = method
        self.uri = uri
        self.headers = headers
        self.data = data
        self.content_type = content_type

    def __str__(self):
        headers = "\n".join("%s: %s" % (k, v) for k, v in self.headers.items())
        return ("Host: %s\nMethod: %s\nUri: %s\nHeaders: %s\nData: %s\n"
                % (self.host, self.method, self.uri, headers, self.data))


class ResponseInternal(object):
    def __init__(self, status=0, headers=None, data=""):
        if headers is None:
            headers = {}
        self.status = status
        self.headers = headers
        self.data = data

    def __str__(self):
        headers = "\n".join("%s: %s" % (k, v) for k, v in self.headers.items())
        return ("Status: %s\nHeaders: %s\nData: %s\n"
                % (self.status, headers, self.data))


class ConnectionConfig(object):
    def __init__(self, req_timeout=60):
        self.conn = Session()
        self.req_timeout = req_timeout
        self.keep_alive = False

    def set_req_timeout(self, req_timeout):
        self.req_timeout = req_timeout

    def is_keep_alive(self):
        return self.keep_alive

    def set_keep_alive(self, flag=True):
        self.keep_alive = flag

    def _request(self, req_inter):
        if self.is_keep_alive():
            req_inter.headers["Connection"] = "Keep-Alive"
        if req_inter.method == 'GET':
            req_inter_url = '%s?%s' % (req_inter.uri, req_inter.data)
            return self.conn.request(req_inter.method, req_inter_url, None, headers=req_inter.headers, timeout=self.req_timeout)
        elif req_inter.method == 'POST':
            if req_inter.content_type == APPLICATION_JSON:
                return self.conn.request(req_inter.method, req_inter.uri,
                                         json=req_inter.data, headers=req_inter.headers, timeout=self.req_timeout)
            elif req_inter.content_type == APPLICATION_FORM:
                return self.conn.request(req_inter.method, req_inter.uri,
                                         data=req_inter.data, headers=req_inter.headers, timeout=self.req_timeout)

        else:
            raise KuaishouVodSDKException("ClientParamsError", 'Method only support GET and POST')

    def send_request(self, req_inter):
        try:
            try:
                http_resp = self._request(req_inter)
            except BadStatusLine:
                LOG.error("keep-alive timeout, reopen connection")
                self.conn.close()
                http_resp = self._request(req_inter)
            headers = dict(http_resp.headers)
            resp_inter = ResponseInternal(status=http_resp.status_code,
                                          headers=headers,
                                          data=http_resp.json())
            if not self.is_keep_alive():
                self.conn.close()
            LOG.debug("GetResponse %s" % resp_inter)
            return resp_inter
        except Exception as ex:
            self.conn.close()
            raise KuaishouVodSDKException("ClientNetworkError", str(ex))
