# -*- coding: UTF-8 -*-

import base64
import ipaddress
import time
import struct
import socket

try:
    import urllib.parse
except ImportError:
    import urlparse
from zlib import crc32
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from Crypto.Random import get_random_bytes
import sys

version = 0x05
network_byte_order = 'big'
encoding = 'utf-8'


class CdnClient(object):
    def __init__(self, key, iv):
        self.key = key
        self.iv = iv

    def sign_cdn_url(self, original_url, client_ip='0.0.0.0', ttl=24 * 3600):
        return self.get_encrypt_url(original_url, 1, 1, "", client_ip, 0, 0, 0, ttl)

    def get_encrypt_conf(self):
        if sys.version_info[0] == 3:
            enc_key = bytes.fromhex(self.key)
            enc_iv = bytes.fromhex(self.iv)
        else:
            enc_key = self.key.decode('hex')
            enc_iv = self.iv.decode('hex')
        return enc_key, enc_iv

    @staticmethod
    def get_ip_version_and_bytes(client_ip):
        if sys.version_info[0] == 3:
            address = ipaddress.ip_address(client_ip)
            ip_int = int(address).to_bytes(length=16, byteorder=network_byte_order)
            if address.version == 6:
                return 1, ip_int
            return 0, ip_int
        else:
            try:
                address_bytes = b'000000000000' + socket.inet_pton(socket.AF_INET, client_ip)
            except:
                pass
            else:
                return 0, address_bytes
            try:
                address_int = socket.inet_pton(socket.AF_INET6, client_ip)
            except:
                pass
            else:
                return 1, address_int
            raise Exception("invalid ip")

    @staticmethod
    def get_uri(original_url):
        if sys.version_info[0] == 3:
            req = urllib.parse.urlparse(url=original_url)
        else:
            req = urlparse.urlparse(url=original_url)
        path = req.path  # type: str
        if "." in path:
            idx = path.find(".")
            path = path[:idx]
        return path

    def compose_plan_text(self, original_url, biz_id, platform, gid,
                          client_ip, limit_speed, limit_time_second, limit_ts_idx,
                          ttl=0):
        bytes_to_encrypt = bytearray()
        ts = int(time.time())
        # 随机4字节整数 [0-4)
        salt = get_random_bytes(4)
        bytes_to_encrypt.extend(salt)

        if sys.version_info[0] == 3:
            # 4字节秒级时间戳 [4, 8)
            bytes_to_encrypt.extend(ts.to_bytes(length=4, byteorder=network_byte_order))
            # 4字节密文生效时间 [8, 12)
            bytes_to_encrypt.extend(ttl.to_bytes(length=4, byteorder=network_byte_order))
            # 2字节业务ID [12, 14)
            bytes_to_encrypt.extend(biz_id.to_bytes(length=2, byteorder=network_byte_order))
            # 1字节平台ID [14, 15)
            bytes_to_encrypt.extend(platform.to_bytes(length=1, byteorder=network_byte_order))
            # 4字节路径CRC [15, 19)
            req_uri = self.get_uri(original_url=original_url)
            bytes_to_encrypt.extend(crc32(req_uri.encode(encoding)).to_bytes(length=4, byteorder=network_byte_order))
            # 2字节限速 [19, 21)
            bytes_to_encrypt.extend(limit_speed.to_bytes(length=2, byteorder=network_byte_order))
            # 2字节限时 [21, 23)
            bytes_to_encrypt.extend(limit_time_second.to_bytes(length=2, byteorder=network_byte_order))
            # 2字节分片限制 [23, 25)
            bytes_to_encrypt.extend(limit_ts_idx.to_bytes(length=2, byteorder=network_byte_order))
            # 2字节gid长度 [25, 27)
            bytes_to_encrypt.extend(len(gid).to_bytes(length=2, byteorder=network_byte_order))
            # gid [27, 27+len(gid))
            bytes_to_encrypt.extend(gid.encode(encoding))
            # 1字节IP版本 [27+len(gid), 28+len(gid))
            ip_version, ip_bytes = self.get_ip_version_and_bytes(client_ip)
            bytes_to_encrypt.extend(ip_version.to_bytes(length=1, byteorder=network_byte_order))
            # 16字节IP [28+len(gid), 44+len(gid))
            bytes_to_encrypt.extend(ip_bytes)
            # 4字节整体CRC [44+len(gid), 48+len(gid))
            total_crc = crc32(bytes_to_encrypt).to_bytes(length=4, byteorder=network_byte_order)
            bytes_to_encrypt.extend(total_crc)
        else:
            # 4字节秒级时间戳 [4, 8)
            bytes_to_encrypt.extend(struct.pack(">I", ts))
            # 4字节密文生效时间 [8, 12)
            bytes_to_encrypt.extend(struct.pack(">I", ttl))
            # 2字节业务ID [12, 14)
            bytes_to_encrypt.extend(struct.pack(">H", biz_id))
            # 1字节平台ID [14, 15)
            bytes_to_encrypt.extend(struct.pack(">B", platform))
            # 4字节路径CRC [15, 19)
            req_uri = self.get_uri(original_url=original_url)
            bytes_to_encrypt.extend(struct.pack(">i", crc32(req_uri)))
            # 2字节限速 [19, 21)
            bytes_to_encrypt.extend(struct.pack(">H", limit_speed))
            # 2字节限时 [21, 23)
            bytes_to_encrypt.extend(struct.pack(">H", limit_time_second))
            # 2字节分片限制 [23, 25)
            bytes_to_encrypt.extend(struct.pack(">H", limit_ts_idx))
            # 2字节gid长度 [25, 27)
            bytes_to_encrypt.extend(struct.pack(">H", len(gid)))
            # gid [27, 27+len(gid))
            bytes_to_encrypt.extend(bytes(gid))
            # 1字节IP版本 [27+len(gid), 28+len(gid))
            ip_version, ip_bytes = self.get_ip_version_and_bytes(client_ip)
            bytes_to_encrypt.extend(struct.pack(">B", ip_version))
            # 16字节IP [28+len(gid), 44+len(gid))
            bytes_to_encrypt.extend(ip_bytes)
            # 4字节整体CRC [44+len(gid), 48+len(gid))
            total_crc = crc32(bytes(bytes_to_encrypt))
            bytes_to_encrypt.extend(struct.pack(">i", total_crc))
        return bytes_to_encrypt

    def gen_pkey(self, original_url, biz_id, platform, gid,
                 client_ip, limit_speed, limit_time_second, limit_ts_idx,
                 ttl=0):
        enc_key, enc_iv = self.get_encrypt_conf()
        aes_cipher = AES.new(key=enc_key, mode=AES.MODE_CBC, iv=enc_iv)
        # 拼接加密字符串
        plain_text = self.compose_plan_text(original_url=original_url, biz_id=biz_id, platform=platform,
                                            gid=gid, client_ip=client_ip, limit_speed=limit_speed,
                                            limit_time_second=limit_time_second, limit_ts_idx=limit_ts_idx,
                                            ttl=ttl)
        # cbc256加密
        cipher_data = aes_cipher.encrypt(pad(plain_text, AES.block_size))
        final_bytes = bytearray()
        # 头部增加2字节版本号
        if sys.version_info[0] == 3:
            final_bytes.extend(version.to_bytes(length=2, byteorder=network_byte_order))
        else:
            final_bytes.extend(struct.pack(">H", version))
        final_bytes.extend(cipher_data)
        # base64加密
        result = base64.urlsafe_b64encode(final_bytes)
        result = result.decode(encoding).rstrip('=')
        return result

    def get_encrypt_url(self, original_url, biz_id, platform, gid,
                        client_ip, limit_speed, limit_time_second, limit_ts_idx,
                        ttl=0):
        pkey = self.gen_pkey(original_url=original_url, biz_id=biz_id, platform=platform, gid=gid,
                             client_ip=client_ip, limit_ts_idx=limit_ts_idx, limit_speed=limit_speed,
                             limit_time_second=limit_time_second, ttl=ttl)
        if sys.version_info[0] == 3:
            segments = urllib.parse.urlsplit(url=original_url)
        else:
            segments = urlparse.urlsplit(url=original_url)
        new_query = 'pkey=' + pkey
        if segments.query:
            new_query = segments.query + '&pkey=' + pkey
        if sys.version_info[0] == 3:
            return urllib.parse.urlunsplit(
                (segments.scheme, segments.netloc, segments.path, new_query, segments.fragment))
        else:
            return urlparse.urlunsplit((segments.scheme, segments.netloc, segments.path, new_query, segments.fragment))
